<?php

class WP_AlterVistaPanel {
	
    private $domain = 'aa.altervista.org';
    private $auth_uri = '';
    private $access_uri = '';
    private $request_uri = '';
    private $redirect_uri;
    private $client_id, $secret;
    public $errno, $error;
    private $last_opt = 'construct';
    private $panel_server_name;

    function __construct($self_authorize = true, $pwd = null) {
	if ((isset($_ENV['AV_IS_DEV']) && $_ENV['AV_IS_DEV']) || (isset($_SERVER['AV_IS_DEV']) && $_SERVER['AV_IS_DEV']) || (isset($_SERVER['REDIRECT_AV_IS_DEV']) && $_SERVER['REDIRECT_AV_IS_DEV'])) {
	    $this->domain = '46.4.17.188';
	}
	$this->auth_uri = 'https://'.$this->domain.'/cgi-bin/auth.pl';
	$this->access_uri = 'https://'.$this->domain.'/cgi-bin/access.pl';
	$this->request_uri = 'https://'.$this->domain.'/cgi-bin/resource.pl';
	$oauth_options = get_option('avopt_oauth_options');
	$this->client_id = $oauth_options['client_id'];
	$this->secret = $oauth_options['secret'];
	$this->access_token = $oauth_options['access_token'];
	$this->refresh_token = $oauth_options['refresh_token'];
	$this->token_type = isset($oauth_options['token_type']) ? $oauth_options['token_type'] : 'Bearer';
	$this->redirect_uri = $oauth_options['redirect_uri'];
	$this->username = AVOPT_ACCOUNT;
	preg_match('/^ns(\d+)/', gethostbyaddr($_SERVER['SERVER_ADDR']), $match);
	$this->panel_server_name = 's'.($match[1]+1).'.altervista.org';
	if(empty($this->client_id) || empty($this->secret) || empty($this->refresh_token)) {
	    $this->reset_oauth_data();
	}
	if(empty($this->client_id) || empty($this->secret)) {
	    return null;
	}
	if(empty($this->access_token)) {
	    $this->refresh_token();
	}
    }
    
    function reset_oauth_data() {
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 15);
	curl_setopt($ch, CURLOPT_TIMEOUT, 25);
	curl_setopt($ch, CURLOPT_URL, 'http://'.$this->panel_server_name.'/oauth2_reset.pl');
	curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
	curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
	curl_setopt($ch, CURLOPT_HEADER, false);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
	curl_setopt($ch, CURLOPT_POST, true);
	curl_setopt($ch, CURLOPT_POSTFIELDS, "nick=".$this->username);
	$res = curl_exec($ch);
	$answ = json_decode($res, true);
	if($answ['ok'] == 1) {
	    $oauth_options = get_option('avopt_oauth_options');
	    $this->client_id = $oauth_options['client_id'];
	    $this->secret = $oauth_options['secret'];
	    $this->access_token = $oauth_options['access_token'];
	    $this->refresh_token = $oauth_options['refresh_token'];
	    $this->token_type = isset($oauth_options['token_type']) ? $oauth_options['token_type'] : 'Bearer';
	    $this->redirect_uri = $oauth_options['redirect_uri'];   
	}
    }
    
    function get_panel_server() {
	return $this->panel_server_name;
    }
    
    function get_sid() {
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 15);
	curl_setopt($ch, CURLOPT_TIMEOUT, 25);
	curl_setopt($ch, CURLOPT_URL, 'http://'.$this->panel_server_name.'/oauth2.pl');
	curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
	curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
	curl_setopt($ch, CURLOPT_HEADER, false);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
	curl_setopt($ch, CURLOPT_POST, true);
	curl_setopt($ch, CURLOPT_POSTFIELDS, "access_token=".$this->access_token);
	$res = curl_exec($ch);
	$sess = json_decode($res, true);
	if ($sess['error'] == "403") {
	    $sid = false;
	    set_transient('avopt_panel_sid', null, 60*30);
	    $this->access_token = null;
	    $oauth_options = get_option('avopt_oauth_options');
	    $oauth_options['access_token'] = $this->access_token;
	    update_option('avopt_oauth_options', $oauth_options);
	    $ret = $this->refresh_token();
	    if($ret) {
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 15);
		curl_setopt($ch, CURLOPT_TIMEOUT, 25);
		curl_setopt($ch, CURLOPT_URL, 'http://'.$this->panel_server_name.'/oauth2.pl');
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
		curl_setopt($ch, CURLOPT_HEADER, false);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_POST, true);
		curl_setopt($ch, CURLOPT_POSTFIELDS, "access_token=".$this->access_token);
		$res = curl_exec($ch);
		$sess = json_decode($res, true);
		if ($sess['error'] == "403") {
		    $sid = false;
		} else {
		    $sid = $sess['sid'];
		    set_transient('avopt_panel_sid', $sid, 60*30);
		}
	    }
	} else {
	    $sid = $sess['sid'];
	    set_transient('avopt_panel_sid', $sid, 60*30);
	}
	return;
    }

    function get_account_info() {
	$this->last_opt = 'get_account_info';
	$data = array(
	    'action' => 'info',
        );
	$res = $this->request($data);
	if($res) {
	    set_transient('avopt_account_info', $res, 36000);
	}
	return;
    }
    
    function get_revenues() {
	$this->last_opt = 'get_revenues';
	$data = array(
	    'action' => 'revenues',
	);
	$res = $this->request($data);
	if($res) {
	    set_transient('avopt_revenues_info', $res, 60*60*6);
	}
	return;
    }
    
    function authorize_user() {
	$this->last_opt = 'authorize_user';
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_HTTPHEADER, array('Host: aa.altervista.org'));
	curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 15);
	curl_setopt($ch, CURLOPT_TIMEOUT, 25);
	curl_setopt($ch, CURLOPT_URL, $this->auth_uri);
	curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
	curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
	curl_setopt($ch, CURLOPT_HEADER, false);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
	curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
	curl_setopt($ch, CURLOPT_USERPWD, "{$this->client_id}:{$this->secret}");
	curl_setopt($ch, CURLOPT_POST, true);
	$client_id = urlencode($this->client_id);
	$password = urlencode($this->password);
	curl_setopt($ch, CURLOPT_POSTFIELDS, "client_id=$client_id&response_type=code&response=auth&state=state&username={$this->username}&password={$password}&redirect_uri={$this->redirect_uri}");
	$res = curl_exec($ch);
	if (curl_errno($ch)) {
		$this->errno = curl_errno($ch);
		$this->error = curl_error($ch);
		$ret = false;
	} else {
	    if(preg_match("/authorization_code=([0-9a-zA-Z]+)/", $res, $matches)) {
		$this->authorization_code = $matches[1];
		return true;
	    } else {
		$this->errno = 1000;
		$this->error = 'User authorization failed';
		$ret = false;
	    }
	}
	return $ret;
    }

    function authorize_client() {
	if(!isset($this->authorization_code)) {
	    $this->authorize_user();
	}
	$this->last_opt = 'authorize_client';
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_HTTPHEADER, array('Host: aa.altervista.org'));
	curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 15);
	curl_setopt($ch, CURLOPT_TIMEOUT, 25);
	curl_setopt($ch, CURLOPT_URL, $this->access_uri);
	curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
	curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
	curl_setopt($ch, CURLOPT_HEADER, false);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
	curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
	curl_setopt($ch, CURLOPT_USERPWD, "{$this->client_id}:{$this->secret}");
	curl_setopt($ch, CURLOPT_POST, true);
	curl_setopt($ch, CURLOPT_POSTFIELDS, "grant_type=authorization_code&code={$this->authorization_code}&redirect_uri={$this->redirect_uri}");
	return $this->access_request($ch);   
    }


    function refresh_token() {
	$this->last_opt = 'refresh_token';
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_HTTPHEADER, array('Host: aa.altervista.org'));
	curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 15);
	curl_setopt($ch, CURLOPT_TIMEOUT, 25);
	curl_setopt($ch, CURLOPT_URL, $this->access_uri);
	curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
	curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
	curl_setopt($ch, CURLOPT_HEADER, false);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
	curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
	curl_setopt($ch, CURLOPT_USERPWD, "{$this->client_id}:{$this->secret}");
	curl_setopt($ch, CURLOPT_POST, true);
	curl_setopt($ch, CURLOPT_POSTFIELDS, "grant_type=refresh_token&refresh_token={$this->refresh_token}");
	$ret = $this->access_request($ch);
	if(!$ret && $this->error == 'invalid_grant') {
	    
	    $this->refresh_token = null;
	    $oauth_options = get_option('avopt_oauth_options');
	    $oauth_options['refresh_token'] = $this->refresh_token;
	    update_option('avopt_oauth_options', $oauth_options);
	    
	} elseif($this->error == 'invalid_client') {
	    $this->reset_oauth_data();
	    $ret = true;
	}
	return $ret;   
    }


    function request($data = array()) {
	$postdata = '';
	foreach ($data as $key => $value) {
	    if($postdata > '') {
		$postdata .= '&';
	    }
	    $postdata .= urlencode($key).'='.urlencode($value);
	}
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 15);
	curl_setopt($ch, CURLOPT_TIMEOUT, 25);
	curl_setopt($ch, CURLOPT_URL, $this->request_uri);
	curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
	curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
	curl_setopt($ch, CURLOPT_HEADER, false);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
	curl_setopt($ch, CURLOPT_HTTPHEADER, array("Host: aa.altervista.org", "Authorization: {$this->token_type} {$this->access_token}"));
	if ($postdata) {
	    curl_setopt($ch, CURLOPT_POST, true);
	    curl_setopt($ch, CURLOPT_POSTFIELDS, $postdata);
	}
	$res = curl_exec($ch);
	if (curl_errno($ch)) {
		$this->errno = curl_errno($ch);
		$this->error = curl_error($ch);
		$ret = false;
	} else {
		$ret = json_decode($res, true);
	}
	curl_close($ch);
	if (isset($ret['error'])) {
	    $this->errno = 1000;
	    $this->error = $ret['error'];
	    error_log($ret['error']);
	    $ret = false;
	    if($this->error == 'invalid_access_token') {
		$this->access_token = null;
		$oauth_options = get_option('avopt_oauth_options');
		$oauth_options['access_token'] = $this->access_token;
		update_option('avopt_oauth_options', $oauth_options);
		$ret = $this->refresh_token();
		if($ret) {
		    $ch = curl_init();
		    curl_setopt($ch, CURLOPT_HTTPHEADER, array('Host: aa.altervista.org'));
		    curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 15);
		    curl_setopt($ch, CURLOPT_TIMEOUT, 25);
		    curl_setopt($ch, CURLOPT_URL, $this->request_uri);
		    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		    curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
		    curl_setopt($ch, CURLOPT_HEADER, false);
		    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		    curl_setopt($ch, CURLOPT_HTTPHEADER, array("Authorization: {$this->token_type} {$this->access_token}"));
		    if ($postdata) {
			curl_setopt($ch, CURLOPT_POST, true);
			curl_setopt($ch, CURLOPT_POSTFIELDS, $postdata);
		    }
		    $res = curl_exec($ch);
		    if (curl_errno($ch)) {
			$this->errno = curl_errno($ch);
			$this->error = curl_error($ch);
			$ret = false;
		    } else {
			$ret = json_decode($res, true);
			if (isset($ret['error'])) {
			    $this->errno = 1000;
			    $this->error = $ret['error'];
			    error_log($ret['error']);
			    $ret = false;
			}
		    }
		    curl_close($ch);
		}
	    }
	}
	return ($ret);
    }

    private function access_request($ch) {
	$res = curl_exec($ch);
	if (curl_errno($ch)) {
		$this->errno = curl_errno($ch);
		$this->error = curl_error($ch);
		$ret = false;
	} else {
		$data = json_decode($res, true);
		if (isset($data['error'])) {
			$this->errno = 1000;
			$this->error = $data['error'];
			error_log($data['error']);
			$ret = false;
		} else {
			if ($data['token_type']) $this->token_type = ucfirst($data['token_type']);
			if ($data['access_token']) $this->access_token = $data['access_token'];
			if ($data['refresh_token']) $this->refresh_token = $data['refresh_token'];
			$oauth_options = get_option('avopt_oauth_options');
			$oauth_options['access_token'] = $this->access_token;
			$oauth_options['refresh_token'] = $this->refresh_token;
			$oauth_options['token_type'] = $this->token_type;
			update_option('avopt_oauth_options', $oauth_options);
			$ret = true;
		}
	}
	curl_close($ch);
	return $ret;
    }

    function clean() {
        delete_option('avopt_oauth_options');
    }
    
}

?>