<?php

// checks posts on status transition

add_action('transition_post_status','avopt_post_check',10,3);
function avopt_post_check($new, $old, $post) {
	//check post status
	if (strcmp($new,'publish') == 0 && strcmp($old,'publish') != 0 && strcmp($post->post_type,'post') == 0) {		
		//check post date
		$wp_url = get_site_url();
		$post_url = get_permalink($post->ID);
		if(strpos($post_url,$wp_url) === false) {
			return;
		}
		avopt_send_post($post->ID);
	}
}
 
 
// checks comments on status transition 
 
add_action('wp_set_comment_status','avopt_comment_check',10,2);
add_action('comment_post','avopt_comment_check',10,2);
function avopt_comment_check($comment_id, $status) {
	
	if (strcmp($status,"approve")==0 or $status == 1)  {
		avopt_send_comment($comment_id);
	}
}

// sends comments to endpoint

function avopt_send_comment($id) {
	
	$comment = get_comment($id);
	$dom = new DOMDocument('1.0', 'UTF-8');
	$root = $dom->createElement('comment');
	$root->appendChild($dom->createElement('author',esc_attr($comment->comment_author)));
	$root->appendChild($dom->createElement('text',esc_attr($comment->comment_content)));
	$root->appendChild($dom->createElement('authorUrl',esc_attr($comment->comment_author_url)));
	$root->appendChild($dom->createElement('type',$comment->comment_type)); 
	$root->appendChild($dom->createElement('permalink',get_permalink($comment->comment_post_ID)));
	$root->appendChild($dom->createElement('blogName',esc_attr(get_bloginfo('name'))));
	$root->appendChild($dom->createElement('postTitle',esc_attr(get_the_title($comment->comment_post_ID))));
	if($_SERVER['AV_SERVER_LANG']) {
		$root->appendChild($dom->createElement('lang',$_SERVER['AV_SERVER_LANG']));
	}
	else {
		$root->appendChild($dom->createElement('lang',$_ENV['AV_SERVER_LANG']));
	}
	$dom->appendChild($root);	
	
	$args = array('xml' => $dom->saveXML());
	//sending post
	$args = array('xml' => $dom->saveXML());
	$url = "http://wp.altervista.org/altervista-optimizer/optimizer.php";
	avopt_async_post($url, $args);
	
}

// sends comments to endpoint

function avopt_send_post($id) {

	$post = get_post($id);
	
	if ($post->post_status != 'publish' && $post->post_status != 'future') return;
	
	
	
	// get the post thumbnail, put it first in the image list
	if (current_theme_supports('post-thumbnails')) {
		if ( has_post_thumbnail($id)) {
			$thumbid = get_post_thumbnail_id($id);
			$att = wp_get_attachment_image_src($thumbid, 'full');
			if (!empty($att[0])) $images[] = $att[0];
		}
	}
	
	// look for any images in the content
	if ( preg_match_all('/<img (.+?)>/', $post->post_content, $matches) ) {
		foreach ($matches[1] as $match) {
			foreach ( wp_kses_hair($match, array('http')) as $attr) 
				$img[$attr['name']] = $attr['value'];
			if ( isset($img['src']) ) {
				if ( isset( $img['class'] ) && false === strpos( $img['class'], 'wp-smiley' ) ) { // ignore smilies
					$images[] = $img['src'];
				}
			}
		}
	}
	
	$dom = new DOMDocument('1.0', 'UTF-8');
	$root = $dom->createElement('post');

	$root->appendChild($dom->createElement('title',esc_attr($post->post_title)));
	$root->appendChild($dom->createElement('excerpt',esc_attr($post->post_excerpt)));
	$root->appendChild($dom->createElement('content',esc_attr($post->post_content)));
	$root->appendChild($dom->createElement('date',$post->post_date));
	
	$author = get_user_meta($post->post_author, 'nickname', true);
	$root->appendChild($dom->createElement('author',$author));
	$root->appendChild($dom->createElement('image',$images[0]));
	$root->appendChild($dom->createElement('permalink',get_permalink($post->ID)));
	$root->appendChild($dom->createElement('blogName',esc_attr(get_bloginfo('name'))));
	if($_SERVER['AV_SERVER_LANG']) {
		$root->appendChild($dom->createElement('lang',$_SERVER['AV_SERVER_LANG']));
	}
	else {
		$root->appendChild($dom->createElement('lang',$_ENV['AV_SERVER_LANG']));
	}
	
	//categories
	$cat_element = $dom->createElement('categories');
	$category_array = get_the_category($id);
	if(!empty($category_array)) {
		foreach($category_array as $category) { 
			$cat_element->appendChild($dom->createElement('category',esc_attr($category->cat_name)));
		}
	}
	$root->appendChild($cat_element);
	
	//tags
	$tag_element = $dom->createElement('tags');
	$tag_array = get_the_tags($id);
	if(!empty($tag_array)) {
		foreach($tag_array as $tag) {
			$tag_element->appendChild($dom->createElement('tag',esc_attr($tag->name)));
		}
	}
	$root->appendChild($tag_element);
	
	$dom->appendChild($root);
	
	$args = array('xml' => $dom->saveXML());
	$url = "http://wp.altervista.org/altervista-optimizer/optimizer.php";
	avopt_async_post($url, $args);

	
}


function avopt_async_post($url, $params)
{
    foreach ($params as $key => &$val) {
      if (is_array($val)) $val = implode(',', $val);
        $post_params[] = $key.'='.urlencode($val);
    }
    $post_string = implode('&', $post_params);

    $parts=parse_url($url);

    $fp = fsockopen($parts['host'], 
        isset($parts['port'])?$parts['port']:80, 
        $errno, $errstr, 1);

    $out = "POST ".$parts['path']." HTTP/1.1\r\n";
    $out.= "Host: ".$parts['host']."\r\n";
    $out.= "Content-Type: application/x-www-form-urlencoded\r\n";
    $out.= "Content-Length: ".strlen($post_string)."\r\n";
    $out.= "Connection: Close\r\n\r\n";
    if (isset($post_string)) $out.= $post_string;

    fwrite($fp, $out);
    fclose($fp);
}

?>