<?php

// Hook into the plugin update check

add_filter('pre_set_site_transient_update_plugins', 'avopt_api_check');
function avopt_api_check( $transient ) {

    // Check if the transient contains the 'checked' information
    // If no, just return its value without hacking it
    if( empty( $transient->checked ) )
        return $transient;
    
    // The transient contains the 'checked' information
    // Now append to it information form your own API
    
    // POST data to send to your API
    $args = array(
        'action' => 'update-check',
        'plugin_name' => AVOPT_SLUG,
        'version' => $transient->checked[AVOPT_BASE_NAME],
    );
    
    // Send request checking for an update
    $response = avopt_api_request( $args );
    
    // If response is false, don't alter the transient
    if( false !== $response ) {
        $transient->response[AVOPT_BASE_NAME] = $response;
    }
    return $transient;
}

// Send a request to the alternative API, return an object

function avopt_api_request( $args ) {

    // Send request
    $request = wp_remote_post( AVOPT_API_URL, array( 'body' => $args ) );
    
    // Make sure the request was successful
    if( is_wp_error( $request )
    or
    wp_remote_retrieve_response_code( $request ) != 200
    ) {
        // Request failed
        return false;
    }
    
    // Read server response, which should be an object
    $response = unserialize( wp_remote_retrieve_body( $request ) );
    
    if( is_object( $response ) ) {
        return $response;
    } else {
        // Unexpected response
        return false;
    }
}


// Hook into the plugin details screen

add_filter('plugins_api', 'avopt_api_information', 10, 3);
function avopt_api_information( $false, $action, $args ) {

    // Check if this plugins API is about this plugin
    if( $args->slug != AVOPT_SLUG ) {
        return false;
    }

    // POST data to send to your API
    $args = array(
        'action' => 'plugin_information',
        'plugin_name' => AVOPT_SLUG,
        'version' => $transient->checked[AVOPT_BASE_NAME],
    );
    
    // Send request for detailed information
    $response = avopt_api_request( $args );

    return $response;
}